/*--- MONTANA.C --------------------------------------------------

	DESQview compatible simple cardgame.

	Demonstrates using of SLICK! in minimum:

	creating windows, using low level video drivers, using mouse
	interface, writing text to windows.

	(c) by MacSoft 1990,1991

*/

#include "slick!.h"
#include <string.h>
#include <stdio.h>
#include <dos.h>
#include <io.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>

#define BUTWAIT 4	/* number of clock ticks to wait on button press */

#define UPBUTTON 0x47		/* attribute for idle button */
#define DOWNBUTTON 0x4f		/* attribute for pressed button */

typedef enum {HEART,DIAMOND,CLUB,SPADE,NONE} kind;
enum {UP,DOWN};

typedef struct {
	char weight;
	char shuffle;
	kind ctype;
} card;

int ccolors[5]={0x070c,0x0704,0x070f,0x0707,0x0202};
char ctitles[]="  234567891JQKA";
int selected;
int usemouse;

char *buttons[]= {
	" New board F1 ",
	" Reshuffle F2 ",
	" Finals    F3 ",
	" Quit      F4 "
};

card table[52];

void waithit(void);		/* wait for key or button press */
void drawcard(int x,int y,int weight,kind t);	/* draw one card */
void mark(int n);		/* mark a card */
int pickone(void);		/* pick a card */
void placecard(int n);	/* reposition a card */
void drawtable(void);	/* redraw the full table */
void cleartable(void);	/* set table to initial order */
void shuffle(void);		/* shuffle non-placed cards */
int finals(void);		/* calcualte final result and clear other cards */
void button(int n, int down);		/* draw the button */

void countto(int ticks)
{
char oldtick,far *ticker=(char far *)0x0000046cL;
	while (ticks--) {
		oldtick=*ticker;
		while (oldtick==*ticker) {
			if (testkey())
				break;
			DVpause();
		}
	}
}

void waithit(void)
{
int x,y,c;
	_mousepress(0,&x,&y,&c);
	while (1) {
		if (testkey()) {
			waitkey();
			return;
		}
		_mousepress(0,&x,&y,&c);
		if (c)
			return;
	}
}

void drawcard(int x,int y,int weight,kind t)
{
char s[4];
	_drawbox(x,y,x+5,y+4,SINGLEFRAME,ccolors[t],1);
	if (t!=NONE) {
		s[0]=t+3;
		s[1]=ctitles[weight];
		if (weight==10) {
			s[2]='0';
			s[3]='\0';
		}
		else
			s[2]='\0';
		_vputs(x+1,y+1,s,ccolors[t]);
		_vputs(x+((weight==10)?2:3),y+3,s,ccolors[t]);
	}
}

void mark(int n)
{
	_vputs((n%13)*6+3,(n/13)*5+3,"<>",0x0f);
}

int pickone(void)
{
int old,mousex,mousey,count;
	mark(selected);
	old=selected;
	while(1) {
		if (old!=selected) {
			drawcard((old%13)*6+1,(old/13)*5+1,table[old].weight,
						table[old].ctype);
			mark(selected);
			old=selected;
		}
		if (testkey()) {
			switch(waitkey()) {
				case 0x3b00:
					return -1;
				case 0x3c00:
					return -2;
				case 0x3d00:
					return -3;
				case 0x3e00:
					return -4;
				case 0x4800:		/* up */
					if (selected>12) {
						selected-=13;
					}
					break;
				case 0x5000:		/* down */
					if (selected<39) {
						selected+=13;
					}
					break;
				case 0x4b00:		/* left */
					if (selected%13!=0)
						--selected;
					break;
				case 0x4d00:		/* right */
					if (selected%13!=12)
						++selected;
					break;
				case 0x1c0d:		/* CR */
					return selected;
			}
		}
		else {
			_mousepress(0,&mousex,&mousey,&count);
			if (count) {
				if (mousey==_videocblk.rows-2) {
					if ((_vgetc(mousex,mousey)>>8)==UPBUTTON) {
						return -1-(mousex-6)/17;
					}
				}
				if (mousey>0 && mousey<21 && mousex>0 && mousex<79) {
					selected=((mousey-1)/5)*13+((mousex-1)/6);
					drawcard((old%13)*6+1,(old/13)*5+1,
						table[old].weight,table[old].ctype);
					mark(selected);
					return selected;
				}
			}
		}
	}
}

void placecard(int n)
{
int i;
	for (i=0;i<52;i++) {
		if (table[i].ctype==NONE) {
			if (i%13==0 && table[n].weight==2)
				break;
			if (i%13==0)
				continue;
			if (table[i-1].weight==table[n].weight-1 &&
					table[i-1].ctype==table[n].ctype)
				break;
		}
	}
	if (i==52) {
		sound(600);
		delay(50);
		nosound();
		return;
	}
	memmove(&table[i],&table[n],sizeof(card));
	table[n].ctype=NONE;
	drawcard((i%13)*6+1,(i/13)*5+1,table[i].weight,table[i].ctype);
	drawcard((n%13)*6+1,(n/13)*5+1,table[n].weight,table[n].ctype);
}

void drawtable(void)
{
int i;
	for (i=0;i<52;i++)
		drawcard((i%13)*6+1,(i/13)*5+1,table[i].weight,table[i].ctype);
}

void cleartable(void)
{
int i;
	for (i=0;i<52;i++) {
		table[i].weight=14-(i%13);
		table[i].ctype=(i/13);
		table[i].shuffle=1;
	}
}

int finals(void)
{
int i,first,row,j,swapp=0;
	for (i=0;i<52;i+=13) {
		first=table[i].ctype;
		row=1;
		for (j=0;j<13;j++) {
			if (table[i+j].ctype==first && table[i+j].weight==j+2 && row==1)
				swapp++;
			else {
				row=0;
				table[i+j].ctype=NONE;
			}
		}
	}
	return (swapp*100)/48;
}

void shuffle(void)
{
int i,j,first,row,swapp=0;
card c;
char t[52];
	for (i=0;i<52;i+=13) {
		first=table[i].ctype;
		row=1;
		for (j=0;j<13;j++) {
			if (table[i+j].ctype==first && table[i+j].weight==j+2 && row==1)
				table[i+j].shuffle=0;
			else {
				table[i+j].shuffle=1;
				row=0;
				t[swapp++]=i+j;
			}
		}
	}
	for (i=0;i<51;i++) {
		if (table[i].shuffle==0)
			continue;
		row=(_myrand()&0x7fff)%(swapp);
		j=t[row];
		t[row]=t[--swapp];
		memmove(&c,&table[j],sizeof(c));
		memmove(&table[j],&table[i],sizeof(c));
		memmove(&table[i],&c,sizeof(c));
	}
}

void button(int n, int down)
{
	n*=20;
	switch (down) {
		case UP:
			prtwnd(n+3,1,buttons[n/20],UPBUTTON);
			prtwnd(n+17,1,"",0x02);
			prtwnd(n+4,2,"",0x02);
			break;
		case DOWN:
			prtwnd(n+3,1,"",0x02);
			prtwnd(n+4,1,buttons[n/20],DOWNBUTTON);
			prtwnd(n+18,1,"",0x02);
			prtwnd(n+4,2,"",0x02);
			countto(BUTWAIT);
	}
}

void interrupt (*oldbreak)(void);

void interrupt newbreak(void)
{
}

void main(void)
{
int one,game,playing=1,count;
char s[81];
	oldbreak=getvect(0x1b);
	setvect(0x1b,newbreak);
	delay(1);
	_videoparm();
	if (!makewnd(0,0,_videocblk.cols-1,_videocblk.rows-4,SINGLEFRAME,ccolors[NONE],80)) goto runoff;
	if (!makewnd(0,_videocblk.rows-3,_videocblk.cols-1,_videocblk.rows-1,"",0x0202,80)) goto runoff;
	cursoroff();
	cleartable();
	drawtable();
	if (!makewnd(19,4,60,19,DOUBLEFRAME,0x0f0f,1)) {
runoff:
		printf("Out of the fu**ing memory!");
		while (closewnd());
		exit(1);
	}
	_mouseinit(STDMOUSE);
	prtcwnd(1,"M O N T A N A  v1.1",0x0f);
	prtcwnd(3,"Using",0x0f);
	prtcwnd(4,wndversio(),0x0f);
	prtcwnd(6,"(c) by MacSoft 1990,1991",0x0f);
	prtcwnd(8,"Thanx to Mr. Aare Tali",0x0f);
	prtcwnd(10,(_mouseinst)?"Press any key or left mouse button":
							"Press any key to continue",0x0f);
	prtcwnd(12,"Dedicated to all FIDOnet Sysops",0x0a);
	waithit();
	closewnd();
	framesoff();
	for (one=0;one<4;one++)
		button(one,UP);
	while (playing) {
		_AH=0x2c;
		__int__(0x21);
		_myseed(_DX);
		cleartable();
		table[0].ctype=NONE;
		table[13].ctype=NONE;
		table[26].ctype=NONE;
		table[39].ctype=NONE;
		shuffle();
		drawtable();
		game=1;
		count=selected=0;
		while (game) {
			one=pickone();
			switch (one) {
				case -1:	/* new game */
					button(0,DOWN);
					game=0;
					button(0,UP);
					break;
				case -2:	/* shuffle */
					button(1,DOWN);
					if (++count<3) {
						shuffle();
						drawtable();
					}
// AT vv
					else {
						if (strcmp(_argv[1], "HahaHa") == 0) {
							shuffle();
							drawtable();
						}
					}
// AT ^^
					button(1,UP);
					break;
				case -3:	/* final score */
					button(2,DOWN);
// AT vv
					{
					card	table1[52];

					memcpy(table1, table, 52 * sizeof(card));
// AT ^^
					game=finals();
					drawtable();
					sprintf(s,"Final score : %d of 100",game);
					makewnd(0,22,79,24,DOUBLEFRAME,0x0f8c,80);
					prtcwnd(0,s,0x8c);
					waithit();
					closewnd();
// AT >>			game=0;
// AT vv
					memcpy(table, table1, 52 * sizeof(card));
					drawtable();
					}
// AT ^^
					button(2,UP);
					break;
				case -4:	/* quit */
					button(3,DOWN);
					game=0;
					playing=0;
					break;
				default:	/* cards... */
					placecard(one);
					break;
			}
		}
	}
	_mousedeinit();
	while (closewnd());
	setvect(0x1b,oldbreak);
}
