/*
** ------ MENUENG.H
**
** (c) by MacSoft 1990, 1991
**
*/

#ifndef __MENUENG__

typedef struct menuengstruc ME_MENU;

/*
** Menu engine and support functions.
*/
#ifdef __cplusplus
extern "C" {
#endif
int far cdecl menueng(int function,...);
ME_MSG far * far cdecl pathfinder(ME_MENU far *menu,int x,int y);
int far cdecl stdmousekey(ME_MENU far *m);
int far cdecl dvmkey(ME_MENU far *m);
int far cdecl cuakey(ME_MENU far *m);
#ifdef __cplusplus
}
#endif

enum { ME_MAKE, ME_MESSAGE };

#ifndef byte
    #define byte unsigned char
#endif
#ifndef word
    #define word unsigned int
#endif

#define ME_MOVE_UP_KEY    0x4800
#define ME_MOVE_DN_KEY    0x5000
#define ME_MOVE_LT_KEY    0x4b00
#define ME_MOVE_RT_KEY    0x4d00
#define ME_ESC_KEY        0x011b
#define ME_CR_KEY         0x1c0d
#define ME_CTRLCR_KEY     0x1c0a
#define ME_BS_KEY         0x0e08
#define ME_HOME_KEY       0x4700
#define ME_END_KEY        0x4f00
#define ME_PAGE_UP_KEY    0x4900
#define ME_PAGE_DN_KEY    0x5100
#define ME_DEL_KEY        0x5300
#define ME_INS_KEY        0x5200
#define ME_TAB_KEY        0x0f09
#define ME_STAB_KEY       0x0f00
#define ME_SPC_KEY        0x3920

#define ME_REDRAW_KEY     0xffff
#define ME_CHIDE_KEY      0xfffe
#define ME_CSHOW_KEY      0xfffd
#define ME_NO_KEY         0xfffc

/*man
NAME
	ME_COLORS - color set structure for menueng

SYNOPSIS
	#include "menueng.h"

DESCRIPTION
	menueng operates on menu structure which contains pointers to
	menu items and to colors for the menu. the structure is defined
	as:

	typedef struct {
	        byte    normcolor,
	                hicolor,
	                barcolor,
	                barhicolor,
	                discolor,
	                disbarcolor;
	} ME_COLORS;

	normcolor	attribute for usual menu text
	hicolor		attribute for highlited character in text
	barcolor	attribute for cursor bar
	barhicolor	attribute for highlited character on cursor
			bar
	discolor	attribute for disabled menu item
	disbarcolor	attribute for cursor bar when on disabled
			item

	usually, a single color structure is used for multiple menu
	item stucture (usually one for the whole application will do).

SEE ALSO
	menueng, ME_ITMES, ME_MENU
*/
typedef struct {
        byte    normcolor,      /* color for normal menu item */
                hicolor,        /* color for highlited character */
                barcolor,       /* color for cursor bar */
                barhicolor,     /* color for higlited character in cursor */
                discolor,       /* color for disabled menu item */
                disbarcolor;    /* cursor bar on disabled item */
} ME_COLORS;


/*man
NAME
	ME_ITEMS - structure for menueng menu items

SYNOPSIS
	#include "menueng.h"

DESCRIPTION
	ME_ITEMS is a structure for single menu item. it's defined as

	typedef struct {
	        int        x,y,len;
	        word       type;
	        int        txtoff,
	                   hioff;
	        word       context;
	        char far * text;
	        void far * func;
        	char far * help;
	}  ME_ITEMS;

	x,y		field start coordinates (screen relative).
	len		length of the item field.
	type		item type bitmask. these bits are currently
			defined:
			0x0001	ME_IT_NORM
				normal item, when this is selected
				menueng just returns with current field
				number set to the item number.
			0x0002	ME_IT_DOOR
				door to new submenu. if this is
				selected, functional pointer is taken
				to be pointer to new menu structure and
				recursive call is made to menueng.
			0x0004	ME_IT_BOOL
				"boolean" item, value is in the next
				bit.this is handled just like FUNC, but
				i'll keep it for compatibility.
			0x0008	ME_IT_BBIT
				value of boolean item (1=true). If you
				have defined a BOOL item, you can keep
				your value here. Radio and check buttons
				keep their states here also.
			0x0010	ME_IT_FUNC
				door to single function. if this is
				selected, functional pointer is taken
				to be pointer to user-defined functions
				which will be called before menu
				processing resumes. this is most often
				used item type in submenues.
			0x0020	ME_IT_DIS
				disabled item. if this bit is set, all
				other bits are ignored and item cannot
				be selected.
			0x0040  ME_IT_RADIOBUTTON
				radio button. radio buttons are grouped
				using the item context number. selecting
				the radio button item turns off all other
				radio buttons with same context. it is
				programmers responsibility to ensure that
				one (and only one) radio button in group
				has ME_IT_BBIT set.
			0x0080	ME_IT_CHECKBUTTON
				check button. if this is selected, BBIT is
				toggled.
			0x00c0	ME_IT_BUTTON
				this is a 'click button' and also serves as
				mask for testing if the item is some kind
				of button. note that check- and radio buttons
				are useable without any handler functions, but
				the 'vanilla' button acts exactly like FUNC
				item. also keep in mind that ME_IT_BUTTON
				has underlying shadow and it extends one line
				down and one column left.
			0x0100	ME_IT_LOCK
				this has effect only whe combined with
				ME_IT_DOOR flag. it is used by keyboard
				handling functions to test if the submenu
				or dialog can be opened at once when the
				mouse cursor is positioned on it (AMIGA and
				Mac style pulldowns) or only when the
				selection is made.
	txtoff		item text offset in field.
	hioff		highlited character offset in item text.
	context		help context number. currently not used by
			menueng, but it is reserved for implementing
			context-sensitive help.
	text		pointer to item text. if the menu item is RADIO-
			or CHECKBUTTON, then the character at offset 2
			is used and BBIT indicator (if the BBIT is set,
			this character is shown. if BBIT is clear, blank
			space is shown instead).
	func		multi-functional pointer. this is interpreted
			by menueng according to the item type.
	help		pointer to short help message or NULL. note
			that if this is set to NULL, help message
			handler will _not_ be called at all. point it
			to empty string if you want to clear the help
			message field. generally, you should set it
			to NULL only if you don't use short help
			messages at all.

SEE_ALSO
	menueng, ME_COLORS, ME_MENU
*/
#define ME_IT_NORM        0x0001
#define ME_IT_DOOR        0x0002
#define ME_IT_BOOL        0x0004
#define ME_IT_FUNC        0x0010
#define ME_IT_DIS         0x0020

#define ME_IT_BBIT        0x0008
#define ME_IT_BUTTON      0x00c0
#define ME_IT_RADIOBUTTON 0x0040
#define ME_IT_CHECKBUTTON 0x0080

#define ME_IT_LOCK        0x0100

typedef struct {
        int     x,y,len;        /* field location and length */
        word    type;           /* menu item type */
        int     txtoff,         /* text offset in field */
                hioff;          /* highlited character offset in text */
        word    context;        /* help context number */
        char far * text;        /* item text */
        void far * func;        /* functional pointer */
        char far * help;        /* pointer to short help notice or NULL */
} ME_ITEMS;

/*man
NAME
	ME_MENU - menueng menu structure

SYNOPSIS
	#include "menueng.h"

DESCRIPTION
	ME_MENU is a data structure on which menueng operates on.
	it is defined as:

	typedef struct menuengstruc ME_MENU;
	struct menuengstruc {
	        word    count,
	                def;
	        word          status;
	        ME_COLORS far *colors;
	        ME_ITEMS far  *items;
	        void far      cdecl (*helpout)(ME_MENU far *);
	        int far cdecl (*startup)(ME_MENU far *,int,int,int,
					int);
	        int far cdecl (*cleanup)(ME_MENU far *);
	        void far cdecl (*errhand)(ME_MENU far *,int);
	        int far cdecl (*scankey)(ME_MENU far *);
	        ME_MENU far *back;
	};

	count		number of items in this menu. at least this
			much items must be defined in items structure.
	def		default item you want the menu cursor to be
			on when menueng is invoked. when menueng runs,
			this will always reflect the number of item
			the cursor is on. this number is 1-based!!!
	status		menu status bits, these define menu behavior.
			0x0001	ME_S_MOVE_UP
				OK to use up arrow for moving to prev.
				entry.
			0x0002	ME_S_MOVE_DN
				OK to use down arrow for moving to
				next entry.
			0x0004	ME_S_MOVE_LT
				OK to use left arrow for moving to
				previous entry.
			0x0008	ME_S_MOVE_RT
				OK to use right arrow for moving to
				next entry.
			0x0010	ME_S_DNEQUCR
				OK to use down arrow as CR.
			0x0020	ME_S_LTRTQT
				OK to quit on left or right arrow
				press.
			0x0040	ME_S_DIALOG
				Dialog. Enables moving with TAB etc.
			0x0080  ME_S_RANGEQ
				Menu engine quits when user tries to move
				below first or past last menu entry.
	colors		pointer to ME_COLORS structure - the color
			set.
	items		pointer to menu items, an array of ME_ITEMS
			structs.
	helpout		pointer to function for writing out short
			help messages (or NULL).
	startup		pointer to menu startup function (or NULL)
	cleanup		pointer to menu cleanup function (or NULL)
	errhand		pointer to error handling function (or NULL)
	scankey		pointer to keyboard input function. the
			following of the keys are treated as special
			by menueng:

			#define-d as	INT 16	description
			--------------  ------  ---------------------
			ME_MOVE_UP_KEY	0x4800	move up
			ME_MOVE_DN_KEY	0x5000	move down
			ME_MOVE_LT_KEY	0x4b00	move left
			ME_MOVE_RT_KEY	0x4d00	move right
			ME_ESC_KEY	0x011b	ESCape
			ME_CR_KEY	0x1c0d	Carrige Return
			ME_BS_KEY	0x0e08	BackSpace
			ME_HOME_KEY	0x4700	Home
			ME_END_KEY	0x4f00	End
			ME_DEL_KEY	0x5300	Del
			ME_INS_KEY	0x5200	Ins
			ME_TAB_KEY	0x0f09	Tab
			ME_STAB_KEY	0x0f00	Shift-Tab
			ME_SPC_KEY	0x3920  Space
			ME_REDRAW_KEY	------	redraw menu (0xffff)
			ME_CHIDE_KEY	------	hide cursor (0xfffe)
			ME_CSHOW_KEY	------	show cursor (0xfffd)
			ME_NO_KEY	------  simple no-op (0xfffc)

	back		internal pointer for linking the menus so
			that the menu creation path can be tracked.
			it has some creative uses to control how menus
			and dialogs behave, but you know what you're doing.
			read examples to see some of the magic.

SEE ALSO
	menueng, ME_COLORS, ME_ITEMS,
*/
#define ME_S_MOVE_UP     0x0001
#define ME_S_MOVE_DN     0x0002
#define ME_S_MOVE_LT     0x0004
#define ME_S_MOVE_RT     0x0008
#define ME_S_DNEQUCR     0x0010
#define ME_S_LTRT_QT     0x0020
#define ME_S_DIALOG      0x0040
#define ME_S_RANGEQ      0x0080
#define ME_S_FAST_UP     0x0100
#define ME_S_FAST_DN     0x0200

#define ME_S_BROWSER     0x0383
#define ME_S_MENUBAR     0x001C
#define ME_S_PULLDOWN    0x0323

#define ME_ERRSTARTUP    1
#define ME_ERRCLEANUP    2

struct menuengstruc {
        word    count,          /* number of items in this menu */
                def;            /* default item selected */
        word    status;         /* status bits ( moves allowed etc.) */
        ME_COLORS far *colors;  /* colors for this item set */
        ME_ITEMS far  *items;   /* items data for this menu */
        void far      cdecl (*helpout)(ME_MENU far *);
            /* pointer to function to write help msgs */
        int far cdecl (*startup)(ME_MENU far *,int,int,int,int);
                                /* pointer to startup function */
        int far cdecl (*cleanup)(ME_MENU far *);
                                /* pointer to cleanup function */
        void far cdecl (*errhand)(ME_MENU far *,int);
                                /* menu system error handler for this menu */
        int far cdecl (*scankey)(ME_MENU far *);
                                /* keyboard input function */
        ME_MENU far *back;      /* points to previous menu */
};

extern ME_MENU far *_topmenu;   /* pointer to first menu created */
extern ME_MENU far *_curntmenu; /* pointer to current menu */

#define __MENUENG__
#endif
