/* --- SLICK!.H -------------------------------------------------
**
** (C) by MacSoft 1990, 1991
**
*/

#ifndef __SLICK!__

#include <dos.h>
#include <stddef.h>

#define SINGLEFRAME "Ŀ "
#define DOUBLEFRAME "ͻ ͼ"
#define HMIXFRAME   "͸ ;"
#define VMIXFRAME   "ķ Ľ"
#define FATFRAME    " "
#define FULLFRAME   " "

#ifndef byte
	#define byte unsigned char
#endif
#ifndef word
	#define word unsigned short
#endif

/*
** Defines DESQview-alike mouse cursor for INT 33 mouse interface.
*/
#define STDMOUSE 0xf000,0x0f04,0,0
enum { LEFT,RIGHT,MIDDLE };
enum { SHOWOBJ,HIDEOBJ,SELECTOBJ };

typedef struct COUNTRY MY_COUNTRY;

/*man*****************************************************************
NAME
	macro definitions

DESCRIPTION
	several useful macros are defined in SLICK!.H.

	cursoroff()	turns off text cursor
	cursoron()	restores cursor size to size before the
			current window was opened
	restorcur()	restores cursor position to what it was
			before current window was opened
	framesoff()	sets viewport size equal to size of window
			frames for current window
	DVpause()	gives away timeslice for other processes
			under DESQview
	DVbeginc()	marks begin of critical timing section
	DVendc()	marks end of critical section
*/
#define cursoroff()   _setcursize(0x2000)
#define cursoron()    _setcursize(_curntwnd->cursize)
#define restorcur()   _setcurpos(_curntwnd->curpos)
#define framesoff()   {_curntwnd->vx1=_curntwnd->x1; \
			   _curntwnd->vy1=_curntwnd->y1; \
			   _curntwnd->vx2=_curntwnd->x2; \
			   _curntwnd->vy2=_curntwnd->y2;}
#define DVpause()  {_AX=0x1000;__int__(0x15);} /* give away timeslice */
#define DVbeginc() {_AX=0x101b;__int__(0x15);} /* begin critical section */
#define DVendc()   {_AX=0x101c;__int__(0x15);} /* end critical section */

/*man*****************************************************************
NAME
	VIDEO, _videocblk - video control structure

SYNOPSIS
	#include "slick!.h"

	extern VIDEO _videocblk

DESCRIPTION
	_videocblk is video system control structure which is used as
	storage for often needed data about video system to speed up
	screen I/O. to fill it up with appropriate values, you could
	call _videoparm, or just use some of the low-level video
	functions. the structure itself is defined as:

	typedef struct {
		word base;
		byte mode;
		byte cols;
		byte rows;
		byte snow;
		byte scan;
		byte ega;
		byte page;
	} VIDEO;

	base	video screen base segment (could be anywhere under
		DESQview or you can point it to your own screen)
	mode	this is a physical video mode at the time when video
		system was initialized. if the physical video mode
		in the BIOS data area is not the same when
		_videoparm is called again, the whole detection will
		be accomplished again.
	cols	screen width
	rows	number of rows on the screen
	snow	flag byte, currently only bit0 is used. when bit0
		is set, only retrace intervals will be used for
		screen access.
	scan	scan lines per text character ( 8 for CGA, 14 for
		MONO or EGA, 16 for MCGA and VGA).
	ega	EGA/VGA mode. this can have following values:
			00 no EGA detected, MONO if mode==7, else CGA
			01 EGA in color mode
			02 EGA in mono mode
			03 PGC color mode
			04 VGA in mono mode
			05 VGA in color mode
			06 MCGA in color mode
			07 MCGA in mono mode
	page	currently visible physical video page

SEE ALSO
	_videoparm
*/
typedef struct {
	word base;              /* physical/virtual screen segment */
	byte mode;              /* physical video mode */
	byte cols;              /* columns on physical/virtual screen */
	byte rows;              /* rows on physical screen */
	byte snow;              /* bit0=1 forces snow check if color mode */
	byte scan;              /* scan lines per character */
	byte ega;               /* 0 - no EGA, 1 - EGA color, 2 - EGA mono */
	byte page;              /* video page */
} VIDEO;

/*
** Screen object structure
*/
typedef struct _screenobject SCROBJ;

struct _screenobject {
	int x,y;                /* corner */
	int cols,rows;          /* size */
	int mode;               /* mode bits */
	int mx,my,mb;           /* mouse coordinates on object or -1,-1 */
				/* if keyboard was used to select and mouse */
				/* button status if mouse was used. */
	word key;               /* IBM PC style keystroke for hotkey */
				/* set to 0 if no hotkey is available */
	int far cdecl (*startup)(SCROBJ far *obj);
				/* if set to NULL, no startup will be done */
	int far cdecl (*objfunc)(SCROBJ far *obj,int message);
				/* if NULL, dummy object will be created */
	int far cdecl (*cleanup)(SCROBJ far *obj);
				/* NULL means no cleanup */
	SCROBJ far * back;
	SCROBJ far * next;      /* make a list of them */
};

#define SO_PRESS     0x0001     /* pressed on object */
#define SO_RELEASE   0x0002     /* released on object */
#define SO_CURSOR    0x0004     /* cursor is on object, don't look buttons */
#define SO_HOLD      0x0008     /* button is held on object */
#define SO_KEYB      0x0100     /* keyboard activity enabled */

/*man********************************************************************
NAME
	WINDOW - window header structure

SYNOPSIS
	#include "slick!.h"

DESCRIPTION
	WINDOW is a structure of window header. each window is divided
	into two major parts - header and saved background. the header
	is defined as

	typedef struct {
		int x1,y1,x2,y2;
		int vx1,vy1,vx2,vy2;
		word curpos,
		     cursize;
		word color;
		void far *last;
		void far *next;
		word cursor;
		word pushcursor;
		SCROBJ far *baseobj;
		word far *hidden;
		word space[1];
	} WINDOW;

	x1,y1,x2,y2 	frame corner coordinates
	vx1,vy1,vx2,vy2	viewport corner coordinates
	curpos		saved cursor position
	cursize		saved cursor size
	color		window attributes (8 high bits for frame, 8
			low bits for viewport)
	last,next	pointers for linked window list
	cursor		current logical cursor position in viewport
	pushcursor	saved cursor position when pushed to another
			level
	baseobj		pointer to first child object, NULL if none
	hidden		pointer to hide buffer or NULL if window is
			visible
	space		beginning of saved background data, NULL if
			none

	when the window is created with makewnd, viewport is set up
	so that frame is 1 char wide, cursor position and size will
	be saved, color is also saved, window is linked into window
	list, baseobj and hidden pointers will be cleared to NULL
	and window background will be saved. if makewnd was allowed
	to draw window frames, they will be drawn; if not, background
	data will be uninitialized (note that makewnd can also use
	pre-allocated buffer for background storage).

SEE ALSO
	makewnd
*/
typedef struct    {
	int x1,y1,x2,y2;        /* frame corners */
	int vx1,vy1,vx2,vy2;    /* viewport corners */
	word curpos,            /* cursor pos & size */
	     cursize;           /* before opening ... */
	word color;             /* used to open */
	void far *last;         /* to previous window */
	void far *next;         /* to next window */
	word cursor;            /* current cursor position */
	word pushcursor;        /* cursor position saved here by pushwnd */
	SCROBJ far *baseobj;    /* base object in window object list */
	word far *hidden;       /* points to data storage for hidden window */
	word space[1];          /* data, expands ... */
} WINDOW;

/*man
NAME
	VMENU - old style pop-up menu structure

SYNOPSIS
	#include "slick!.h"

DESCRIPTION
	VMENU structure is used for creating menus with mkhmenu,
	it's still here just for compatibility with older versions,
	in these days, you should use menueng. anyway, here's a
	layout:

	typedef struct {
		int makenew     :1;
		int center      :1;
		int leavemenu   :1;
		char far *frame;
		word wndcolor;
		byte normcolor;
		byte invcolor;
		byte hicolor;
		int defchoice;
		int reserved;
	} VMENU;

	makenew		if set to 1, mkhmenu will create a new window
			before creating menu
	center		set to 1 if you want items centered into frames
	leavemenu	set to 1 if you want mkhmenu to leave the menu
			window open when it returns
	frame,wndcolor	frame characters and color for makewnd
	normcolor	attribute for menu items
	invcolor	color for menu bar
	hicolor		color for highlited keys
	defchoice	item number you want cursor to be on initially
	reserved	empty space on each side of the longest item

SEE ALSO
	mkhmenu
*/
typedef struct {
	int makenew     :1;     /* 1 for open window, 0 if exist on screen */
	int center      :1;     /* 1 if you want items centered */
	int leavemenu   :1;     /* 1 if you don't want menu closed on exit */
	char far *frame;        /* frame for window */
	word wndcolor;          /* color for window (frame&fill) */
	byte normcolor;         /* color for normal items */
	byte invcolor;          /* color for selection bar */
	byte hicolor;           /* color for hot keys */
	int defchoice;          /* item selected by default when entering */
	int reserved;           /* empty space on each side of item */
} VMENU;

/*man
NAME
	MEITEM - raw menu item structure

SYNOPSIS
	#include "slick!.h"

DESCRIPTION
	this is menu item structure as needed for rawmenu. it's
	preserved only for compatibility, but here it is:

	typedef struct {
		int x,y;
		int xlen;
		char far *text;
		byte normcolor;
		byte invcolor;
		byte hicolor;
		int textoff;
		int highoff;
	} MEITEM;

	x,y		absolute screen coordinates for this item
	xlen		length of the item field
	text		item text
	normcolor	attribute for normal items
	invcolor	attribute for cursor bar when on this item
	textoff		text offset in field
	highoff		offset to highlited character in the text

SEE ALSO
	rawmenu
*/
typedef struct {
	int x,y;                /* absolute screen coordinates for cursor */
	int xlen;               /* field length */
	char far *text;         /* menu selection text */
	byte normcolor;         /* color for normal text */
	byte invcolor;          /* color for cursor bar */
	byte hicolor;           /* color for highlited character */
	int textoff;            /* text offset in cursor bar */
	int highoff;            /* offset in string for highlited character */
} MEITEM;                       /* not to be confused with ME_ITEM ! */

/*man
NAME
	ME_MSG - message structure

SYNOPSIS
	#include "slick!.h"

DESCRIPTION
	ME_MSG is a struture of message used by primitive message
	passing system. It's defined as

	typedef struct menumsgstruc ME_MSG;
	struct menumsgstruc {
		int index;
		word far *message;
		void far cdecl (*eofmsg)(ME_MSG far *);
	};

	index		this is offset to the next key in message,
			normally set to 0 before sending the message
	message		this is an array of INT 16 style keystroke
			codes or in fact anything your program knows
			how to deal with. message end is marked
			with 0.
	eofmsg		this is a pointer to function which will be
			called when the last key is read from the
			message. if it is NULL, nothing will be done.
			it is useful when you send message in
			dynamically allocated space (like pathfinder
			does)

SEE ALSO
	sendkeymsg, pathfinder
*/
typedef struct menumsgstruc ME_MSG;
struct menumsgstruc {
	int index;              /* index in message */
	word far *message;      /* message data */
	void far cdecl (*eofmsg)(ME_MSG far *);
				/* call when message is over */
};

/*
** These are very important variables, you can freely read them but
** overwriting them may cause very funny effects...
*/
extern WINDOW far *_curntwnd;                /* active window pointer */
extern SCROBJ far *_scrbaseobj;              /* base object for screen */
extern void far *_virtwndbuf;                /* virtual window buffer */
extern VIDEO _videocblk;                     /* video control block */
extern ME_MSG far *_curntmsg;                /* pointer to pending message */
extern byte _mouseinst,mousedvparm;          /* mouse status & DV param. */
extern void far cdecl (*_showhook)(void);    /* hook to show mouse cursor */
extern void far cdecl (*_hidehook)(void);    /* hook to hide mouse cursor */
extern void far cdecl (*_hideobjects)(void); /* hook to hide window objects */
extern void far cdecl (*_killobjects)(void); /* hook to kill window objects */
extern int far cdecl (*_mapuphook)(int c);   /* character mapping function */
extern int far cdecl (*_countrymap)(void);   /* pointer to COUNTRY mapper */

/*
** Function prototypes
*/
#ifdef __cplusplus
extern "C" {
#endif
/*
** Screen objects.
*/
/*
 Generic objects.
 */
int far cdecl addobj(SCROBJ far *base,SCROBJ far *newobj);
int far cdecl delobj(SCROBJ far *obj);
int far cdecl showobj(SCROBJ far *obj);
int far cdecl hideobj(SCROBJ far *obj);
/*
 Screen-oriented objects
 */
int far cdecl addscrobj(SCROBJ far *newobj);
int far cdecl delscrobj(SCROBJ far *obj);
/*
** Window-oriented objects
*/
int far cdecl addwndobj(SCROBJ far *newobj);
int far cdecl delwndobj(SCROBJ far *obj);
void far cdecl hideallwndobj(void);
void far cdecl delallwndobj(void);
/*
** Memory management.
*/
void far * far cdecl _mymalloc(unsigned long nbytes);
void far cdecl _myfree(void far *adr);
void far cdecl _myfail(word got,word wanted);
unsigned long far cdecl _mycoreleft(void);
/*
** User timer suppport.
*/
void interrupt far _timer(void);
void far cdecl _inittimer(int count,word far *where);
void far cdecl _deinittimer(void);
void far cdecl _eventtimer(void far cdecl (*func)(int channel));
/*
** Keyboard.
*/
word far cdecl waitkey(void);
word far cdecl testkey(void);
/*
** Country support
*/
int far cdecl _mapupascii(int c);
int far cdecl _mapupalter(int c);
int far cdecl _mygetcountry(int code,MY_COUNTRY far *co);
/*
** Message passing
*/
int far cdecl sendkeymsg(ME_MSG far *msg);
int far cdecl getkeymsg(void);
void far cdecl killkeymsg(void);
void far cdecl pushkey(word keycode);
/*
** Generic functions.
*/
int far cdecl DVactive(void);
int far cdecl vidmode(int newmode);
char far * far cdecl wndversio(void);
/*
** Fast pseudo-random generator.
*/
word far cdecl _myrand(void);
word far cdecl _myseed(word newseed);
/*
** Screen management.
*/
WINDOW far * far cdecl slowvputc(WINDOW far *base,int x,int y,
				word charattr);
WINDOW far * far cdecl slowvgetc(WINDOW far *base,int x,int y,
				word far *charattr);
int far cdecl rawmenu(int items,int def,MEITEM menu[],int leavebar,
				word far cdecl (*scan)(int curnt));
int far cdecl putcwnd(int character);
int far cdecl putswnd(char far *s);
int far cdecl shrnkwnd(int step);
int far cdecl statwnd(char far *s,char far *frame,word color);
int far cdecl pushwnd(void);
int far cdecl popwnd(void);
void far cdecl snowtest(int flag);
int far cdecl makemenu(int x1,int y1,VMENU far *flags,
				 char far *items,word far cdecl (*scan)(int curnt));
int far cdecl getswnd(int x1,int y1,char far *prompt,char far *dest,
				char far *frame,word pattr,int len,
				word ipad,word far cdecl (*scan)(void));
int far cdecl getwinp(int x1,int y1,word pad,char far *dest,
                int len,word far cdecl (*scan)(void));
int far cdecl locate(int x1,int y1);
int far cdecl clearwnd(word charattr);
int far cdecl scrollup(word charattr);
int far cdecl scrolldn(word charattr);
int far cdecl prtcwnd(int y1,char far *s,int att);
int far cdecl prtwnd(int x1,int y1,char far *s,int att);
int far cdecl makewnd(int x1,int y1,int x2,int y2,char far *frame,
                      word color,int zoom);
int far cdecl closewnd(void);
int far cdecl forgetwnd(void);
int far cdecl _editline(int x1,int y1,word pad,char far *dest,
	      int len,word far *lastkey,word far cdecl (*scan)(int insert));
void far cdecl _drawbox(int x1,int y1,int x2,int y2,char far *frame,
				  word color,int infill);
/*
** Low level video drivers, assembly language.
*/
void far cdecl _recolor(int x,int y,word len,int att);
void far cdecl _vputs(int x,int y,char far *s,int att);
void far cdecl _vputsc(int x,int y,char far *s,int a,word maxlen);
void far cdecl _vputscp(int x,int y,char far *s,int a,word maxlen);
void far cdecl _tovid(int x,int y,word far *s,word wcount);
void far cdecl _fromvid(int x,int y,word far *s,word wcount);
void far cdecl _vlwrite(int x,int y,word char_attr,word wcount);
void far cdecl _vputc(int x,int y,word char_attr);
word far cdecl _vgetc(int x,int y);
word far * far cdecl _scrbase(void);
word far * far cdecl _videoparm(void);
void far cdecl _setcurpos(word pos);
void far cdecl _setcursize(word size);
int far cdecl _getcurpos(void);
int far cdecl _getcursize(void);
/*
** Low level mouse drivers, assembly language, all in one module.
*/
int far cdecl _mouseinit(word andmask,word xormask,int x,int y);
void far cdecl _mousedeinit(void);
void far cdecl _mouseshow(void);
void far cdecl _mousehide(void);
int far cdecl _mousestatus(int far *x,int far *y);
int far cdecl _mousepress(int button, int far *x,int far *y,int far *cnt);
int far cdecl _mouserelease(int button, int far *x,int far *y,int far *cnt);
void far cdecl _mouseimg(word andmask,word xormask);
void far cdecl _mousepos(int newx,int newy);
#ifdef __cplusplus
}
#endif

#define __SLICK!__

#endif
